@@ -249,8 +249,14 @@ class Chat():
             The chat ID
         """
         try:
+            # Extract content from Message object if it is one
+            if isinstance(system_prompt, Message):
+                system_content = system_prompt.content
+            else:
+                system_content = str(system_prompt)
+            
             system_message = Message(
-                content=str(system_prompt),
+                content=system_content,
                 role="system",
                 provider=self._model.provider,
                 model=self._model.model
@@ -546,12 +552,21 @@ class Chat():
         messages : List[Dict[str, Any]]
             Messages to store
         response_content : str
-            Assistant response content
+            Assistant response content (only the message body)
         """
         try:
+            # Extract user content - handle both text and multimodal (text + image)
+            user_content = messages[-1]["content"]
+            if isinstance(user_content, list):
+                # Multimodal content - extract only the text part
+                user_text = next((item.get("text", "") for item in user_content if item.get("type") == "text"), "")
+            else:
+                # Simple text content - ensure it's a string
+                user_text = str(user_content) if user_content else ""
+            
             # Create Message dataclasses for the last two messages (user + assistant)
             user_message = Message(
-                content=messages[-1]["content"],
+                content=user_text,
                 role=messages[-1]["role"],
                 provider=self._model.provider,
                 model=self._model.model
@@ -564,6 +579,7 @@ class Chat():
                 model=self._model.model
             )
             
+            # Store as Message objects, let MongoDB handle the conversion
             self._history.store_chat(self.chat_id, [user_message, assistant_message])
         except Exception as e:
             logger.error(f"Failed to store messages: {e}")
@@ -671,18 +687,24 @@ class Chat():
             # Make streaming API call
             response = self._model.ask_stream(messages, metadata=metadata)
 
+            # Variable to store the complete response from ModelStreamTail
+            full_response_content = ""
+
             async for chunk in response:
                 if (isinstance(chunk, ModelStreamHead)):
                     yield ChatStreamHead(chat_id=self.chat_id, **asdict(chunk)) 
                 elif (isinstance(chunk, ModelStreamChunk)):
                     yield ChatStreamChunk(**asdict(chunk))
                 elif (isinstance(chunk, ModelStreamTail)):
+                    # Extract the complete response from ModelStreamTail
+                    full_response_content = chunk.response
                     yield ChatStreamTail(**asdict(chunk))
                 else:
                     yield chunk
             
-            # Store messages using Message dataclasses
-            #self._store_messages(messages, response_text)
+            # Store messages with the complete response from ModelStreamTail
+            if full_response_content:
+                self._store_messages(messages, full_response_content)
             
         except Exception as e:
             logger.error(f"Stream request failed: {e}")
